@@ -1,7 +1,7 @@
 package Web::Query;
 our $AUTHORITY = 'cpan:TOKUHIROM';
 # ABSTRACT: Yet another scraping library like jQuery
-$Web::Query::VERSION = '0.35';
+$Web::Query::VERSION = '0.36';
 use strict;
 use warnings;
 use 5.008001;
@@ -12,8 +12,9 @@ use HTML::Selector::XPath 0.06 qw/selector_to_xpath/;
 use Scalar::Util qw/blessed refaddr/;
 use HTML::Entities qw/encode_entities/;
 
-use List::MoreUtils qw/ uniq /;
+use List::Util qw/ reduce uniq /;
 use Scalar::Util qw/ refaddr /;
+
 our @EXPORT = qw/wq/;
 
 our $RESPONSE;
@@ -57,6 +58,8 @@ sub new {
 sub _resolve_new {
     my( $class, $stuff, $options) = @_;
 
+    return $class->new_from_element([],undef,$options) unless defined $stuff;
+
     if (blessed $stuff) {
         return $class->new_from_element([$stuff],undef,$options)
             if $stuff->isa('HTML::Element');
@@ -114,9 +117,12 @@ sub new_from_html {
 }
 
 sub new_from_element {
-    my $class = shift;
+    my $self_or_class = shift;
+
     my $trees = ref $_[0] eq 'ARRAY' ? $_[0] : [$_[0]];
-    return bless { trees => [ @$trees ], before => $_[1] }, $class;
+
+    return bless { trees => [ @$trees ], before => $_[1] }, 
+        ref $self_or_class || $self_or_class;
 }
 
 sub end {
@@ -181,6 +187,7 @@ sub contents {
 
 sub as_html {
     my $self = shift;
+    my %args = @_;
 
     my @html = map {
         ref $_ ? ( $_->isa('HTML::TreeBuilder::XPath::TextNode') || $_->isa('HTML::TreeBuilder::XPath::CommentNode' ) )
@@ -189,6 +196,8 @@ sub as_html {
                : $_ 
     } @{$self->{trees}};
 
+    return join $args{join}, @html if defined $args{join};
+
     return wantarray ? @html : $html[0];
 }
 
@@ -225,7 +234,9 @@ sub text {
         return $self;
     }
 
-    my @html = map { $_->as_text } @{$self->{trees}};
+    my @html = map { 
+        ref $_ ? $_->as_text : $_
+    } @{$self->{trees}};
     return wantarray ? @html : $html[0];
 }
 
@@ -579,7 +590,7 @@ sub add {
 
     my %ids = map { $self->_node_id($_) => 1 } @{ $self->{trees} };
 
-    ( ref $self )->new_from_element( [ 
+    $self->new_from_element( [ 
         @{$self->{trees}}, grep { ! $ids{ $self->_node_id($_) } } @nodes  
     ], $self );
 }
@@ -606,6 +617,20 @@ sub next {
     return (ref $self || $self)->new_from_element(\@new, $self);
 }
 
+sub match {
+    my( $self, $selector ) = @_;
+
+    my $xpath = ref $selector ? $$selector : selector_to_xpath($selector);
+
+    my $results = $self->map(sub{
+            my(undef,$e) = @_;
+        return 0 unless ref $e;  # it's a string
+        return !!$e->get(0)->matches($xpath);
+    });
+
+    return wantarray ? @$results : $results->[0];
+}
+
 sub not {
     my( $self, $selector ) = @_;
 
@@ -639,6 +664,41 @@ sub next_until {
     return $collection;
 }
 
+sub split {
+    my( $self, $selector, %args ) = @_;
+
+    my @current;
+    my @list;
+
+    $self->contents->each(sub{
+            my(undef,$e)=@_;
+
+            if( $e->match($selector) ) {
+                push @list, [ @current ];
+                @current = ( $e );
+            }
+            else {
+                if ( $current[1] ) {
+                    $current[1] = $current[1]->add($e);
+                }
+                else {
+                    $current[1] = $e;
+                }
+            }
+    });
+    push @list, [ @current ];
+
+    if( $args{skip_leading} ) {
+        @list = grep { $_->[0] } @list;
+    }
+
+    unless ( $args{pairs} ) {
+        @list = map { reduce { $a->add($b) } grep { $_ } @$_ } @list;
+    }
+
+    return @list;
+}
+
 sub last_response {
     return $RESPONSE;
 }
@@ -664,7 +724,7 @@ Web::Query - Yet another scraping library like jQuery
 
 =head1 VERSION
 
-version 0.35
+version 0.36
 
 =head1 SYNOPSIS
 
@@ -714,7 +774,13 @@ This is a shortcut for C<< Web::Query->new($stuff) >>. This function is exported
 
 =item my $q = Web::Query->new($stuff, \%options )
 
-Create new instance of Web::Query. You can make the instance from URL(http, https, file scheme), HTML in string, URL in string, L<URI> object, and instance of L<HTML::Element>.
+Create new instance of Web::Query. You can make the instance from URL(http, https, file scheme), HTML in string, URL in string, L<URI> object, C<undef>, and either one 
+L<HTML::Element> object or an array ref of them.
+
+    # all valid creators
+    $q = Web::Query->new( 'http://techblog.babyl.ca' );
+    $q = Web::Query->new( '<p>foo</p>' );
+    $q = Web::Query->new( undef );
 
 This method throw the exception on unknown $stuff.
 
@@ -834,9 +900,17 @@ Return the last matching element.
 
 This method constructs a new Web::Query object from the last matching element.
 
+=head3 match($selector)
+
+Returns a boolean indicating if the elements match the C<$selector>. 
+
+In scalar context returns only the boolean for the first element.
+
+For the reverse of C<not()>, see C<filter()>.
+
 =head3 not($selector)
 
-Return all the elements not matching the C<$selector>.
+Returns all the elements not matching the C<$selector>.
 
     # $do_for_love will be every thing, except #that
     my $do_for_love = $wq->find('thing')->not('#that');
@@ -921,9 +995,16 @@ The content can be anything accepted by L</new>.
 
 =head3 as_html
 
-Return the elements associated with the object as strings. 
-If called in a scalar context, only return the string representation
-of the first element.
+Returns the string representations of either the first or all elements,
+depending if called in list or scalar context.
+
+If given an argument C<join>, the string representations of the elements
+will be concatenated with the given string.
+
+    wq( '<div><p>foo</p><p>bar</p></div>' )
+        ->find('p')
+        ->as_html( join => '!' );
+    # <p>foo</p>!<p>bar</p>
 
 =head3 C< attr >
 
@@ -1104,7 +1185,19 @@ You can specify your own instance of L<LWP::UserAgent>.
 
     $Web::Query::UserAgent = LWP::UserAgent->new( agent => 'Mozilla/5.0' );
 
-=head1 TROUBLESHOOTING
+=head1 FAQ AND TROUBLESHOOTING
+
+=head2 How to find XML processing instructions in a document?
+
+It's possible with L<Web::Query::LibXML> and by using an xpath expression
+with C<find()>:
+
+    # find <?xml-stylesheet ... ?>
+    $q->find(\"//processing-instruction('xml-stylesheet')");
+
+However, note that the support for processing instructions 
+in L<HTML::TreeBuilder::LibXML::Node> is sketchy, so there
+are methods like C<attr()> that won't work.
 
 =head2 Can't get the content of script elements
 
@@ -1143,7 +1236,13 @@ Tokuhiro Matsuno E<lt>tokuhirom AAJKLFJEF@ GMAIL COME<gt>
 
 =head1 SEE ALSO
 
-L<pQuery>
+=over
+
+=item L<pQuery>
+
+=item L<XML::LibXML::jQuery>
+
+=back
 
 =head1 LICENSE
 
